function [w,CPU_TIME] = TSLDA(Data, Class,EXP)
%--------------------------------------------------------------------------
% TSLDA Two Stage Linear Discriminant Analysis 
%
%   [w,CPU_TIME]  = TSLDA(Data, Class, EXP)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%     EXP:   type of exponential function for between-class scatter. Two
%            types are available:
%                                'exp1': a*exp(b*j);
%                                'exp2': a*exp(b*j)+c*exp(d*j); 
%            Default is set as 'exp2'
%     
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------

t0 = cputime;

if nargin==2
    EXP={'exp2'};
end

%------------------------------ START -------------------------------%
c        = max(Class); % number of classes
[Nvec,dim] = size(Data);
mu    = sum(Data)/Nvec;

for i = 1:c
	loc     = find(Class==i);
	[nvec(i),cl] = size(loc);
end

% STEP 1: Pre-processing stage
Ht = (Data - ones(Nvec,1)*mu)';
[Ut,Dt] = svd(Ht,0);
rt=rank(Dt);
Ut = Ut(:,1:rt);
m = Ut'*Data';
m=m';
mu = mean(m);
Hb=[]; Hw=[];
for i=1:c
    rg = 1+sum(nvec(1:i-1)):sum(nvec(1:i));
    x = m(rg,:);
    mui = mean(x);
    Hb = [Hb, sqrt(nvec(i))*(mui-mu)'];
    Hw = [Hw, (x - ones(nvec(i),1)*mui)'];
end
rb = rank(Hb);
rw = rank(Hw);
rb = rank(Hb);

% Step 2: Estimation of non-singular scatter matrices
[Uw,Dw] = svd(Hw);
Dw = Dw(1:rw,1:rw);
dw=diag(Dw);

[Ub,Db] = svd(Hb);
Db = Db(1:rb,1:rb);
db=diag(Db);

%### Extrapolation of Sw ###
Per = 50;
for j=1:rw
    p = sum(dw(1:j))/sum(dw)*100;
    if p>Per
        break
    end
end
rw_new = j;
xn=1:rw_new; % parameters for cftool
xn=xn'; % parameters for cftool
dw_new = dw(1:rw_new); % parameters for cftool
parm = fit(xn,dw_new,'exp2');
for j=rw_new+1:rt
    dw(j)=parm.a*exp(parm.b*j)+parm.c*exp(parm.d*j);
end

%### Extrapolation of Sb ###
Per = 50;
for j=1:rb
    p = sum(db(1:j))/sum(db)*100;
    if p>Per
        break
    end
end
rb_new = j;
xb=1:rb_new; % parameters for cftool
xb=xb'; % parameters for cftool
db_new = db(1:rb_new); % parameters for cftool
parm = fit(xb,db_new,char(EXP)); % either exp1 or exp2

if strcmpi(EXP,'exp1')==1
    for j=rb_new+1:rt
        db(j)=parm.a*exp(parm.b*j);                      % exp1
    end   
elseif strcmpi(EXP,'exp2')==1
    for j=rb_new+1:rt
        db(j)=parm.a*exp(parm.b*j)+parm.c*exp(parm.d*j); % exp2
    end    
end


% Step 3: Computation of W1

Dw = diag(dw);
DwInv = inv(Dw);

% Compute orientation w1: Sw^-1 * Sb 
R = Uw'*Hb;
R = (DwInv^2)*R;
R = Uw*R;
R = R*Hb';
[w1,Del] = eig(R);
dh = diag(Del);
[Eval,inx] = sort(dh,'descend');
w1=w1(:,inx(1:rb));

%w1 = w1(:,1:rb);

% Step 4: Computation of W2
Db = diag(db);
DbInv = inv(Db);

% Compute orientation w2: Sb^-1 * Sw
R = Ub'*Hw;
R = (DbInv^2)*R;
R = Ub*R;
R = R*Hw';
[w2,Del] = eig(R);
dh = diag(Del);
[Eval,inx] = sort(dh,'descend');
w2=w2(:,inx(1:rb));

% Step 5
w = [w1,w2];
w = Ut*w;
%------------------------------  END  -------------------------------%

CPU_TIME = cputime - t0;
end

