function [G,TIME]  = ULDA(Data, Class)
%--------------------------------------------------------------------------
% ULDA Uncorrelated Linear Discriminant Analysis 
%
%   [w,CPU_TIME]  = ULDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%   NOTE: This code is a modified version of Ye's original ULDA code
%
%--------------------------------------------------------------------------
 
t0 = cputime;
k        = max(Class); % number of classes

%-------------------------------------------------------------------------
[m,n] = size(Data);
cc    = sum(Data)/m;

for i = 1:k 
	loc                  = find(Class==i);
	[num,o]              = size(loc);

	if num==1
		B(loc,1:n)   = 0;
		B(m+i,1:n)   = 0;
	else
		TMP          = sum(Data(loc,1:n))/num;
               
		B(i,1:n)     = sqrt(num)*(TMP - cc);
		B(loc+k,1:n) = Data(loc,1:n) - ones(num,1)*TMP;
	end;
end;

Hw = B(k+1:k+m, :)';

Hb = B(1:k, :)';

Size_low = rank(Hb);

Ht = (Data - ones(m,1)*cc)';

[U1,D1] = svd(Ht,0);     % Ht = U1 D1 V1'
s = rank(Ht);
                                                                                                      
D1 = D1(1:s, 1:s);
U1 = U1(:,1:s);
                                                                                                      
d1 = diag(D1);
d1 = 1./d1;
D1 = diag(d1);

B = D1*U1'*Hb;

[P,D2] = svd(B,0);

X = U1*D1*P;

G = X(:, 1:Size_low);
TIME = cputime - t0;
